/* This may look like C++ code, but it is really -*- C -*-
 *
 * xdialdctl.c
 *
 * An Xaw application for controlling diald
 *
 * $Id: xdialdctl.c,v 1.5 1996/12/05 21:09:24 joe Exp $
 * Copyright 1996 Joe B. Edmonds (joe@elem.com)
 *
 */

#include <stdio.h>
#include <limits.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <sys/param.h>
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Xaw/Box.h>
#include <X11/Xaw/Command.h>
#ifdef TRY_PACKETS_SCROLLBAR
#include <X11/Xaw/Scrollbar.h>
#endif

#define QUITXDIALDCTL_COLOR "red"
#define BG_IDLE_COLOR "white"
#define BG_FORCED_COLOR "yellow"
#define BG_CONNECTED_COLOR "green"
/* this is probably safely long enough, but I have no proof of this */
#define DIALD_MONITOR_LINE_LENGTH (1024)

#define SAMESTRING(x,y) (strcmp((x),(y))==0?1:0)
#define DIFFERENTSTRING(x,y) (strcmp((x),(y))==0?0:1)

/* diald fifo */
FILE *diald_fifo;
char fifo_string[100];
char monitor_name[MAXPATHLEN] = "/tmp/xdialdctl_fifo";
char *force_string = "force\n";
char *unforce_string = "unforce\n";
char *connect_string = "up\n";
char *disconnect_string = "down\n";
FILE *monitor_fifo;
char ttl_string[] = "TTL: 00:00:00";
/* xxx it would be really nice to be able to set this properly! */
int is_forced = 0;
/* whether the link is currently up */
int up_down=0;
/* various colors */
Pixel quitxdialdctl_color, bg_idle_color, bg_forced_color, bg_connected_color;
/* widgets */
Widget toplevel, slipon, slipoff, box, quitxdialdctl, ttl
#ifdef TRY_PACKETS_SCROLLBAR
,inpackets, outpackets
#endif
;

/* Xt arguments structures */
Arg sliponArgs[] = {
  {XtNx,	10},
  {XtNy,	40},
	{XtNshapeStyle,	XawShapeOval},
  {XtNlabel,	(int)" Force "}};

Arg slipoffArgs[] = {
  {XtNx,	50},
  {XtNy,	40},
	{XtNshapeStyle,	XawShapeOval},
  {XtNlabel,	(int)"Disconnect"}};

Arg quitxdialdctlArgs[] = {
  {XtNx,	50},
  {XtNy,	40},
	{XtNshapeStyle,	XawShapeOval},
	{XtNbackground, (int)NULL},
  {XtNlabel,	(int)"Quit"}};

Arg boxArgs[] = {
  {XtNorientation,	(int)XtEhorizontal},
	{XtNbackground, (int)NULL}};

Arg ttlArgs[] = {
  {XtNx,	50},
  {XtNy,	40},
  {XtNlabel,	(int)ttl_string}};

#ifdef TRY_PACKETS_SCROLLBAR
Arg inpacketsArgs[] = {
	{XtNwidth, 50},
	{XtNheight, 20},
	{XtNorientation, (int)XtorientHorizontal}
};
#endif	

void Redisplay()
{
	if (up_down) {
		XtVaSetValues(box,XtNbackground,bg_connected_color,NULL);
		XtVaSetValues(slipoff,XtNlabel,"Disconnect",NULL);
	}
	else {
		XtVaSetValues(box,XtNbackground,bg_idle_color,NULL);
		XtVaSetValues(slipoff,XtNlabel," Connect  ",NULL);
	}
	if (is_forced) {
		XtVaSetValues(slipon,XtNbackground,bg_forced_color,NULL);
		XtVaSetValues(slipon,XtNlabel,"Unforce",NULL);
	}
	else {
		XtVaSetValues(slipon,XtNbackground,bg_idle_color,NULL);
		XtVaSetValues(slipon,XtNlabel," Force ",NULL);
	}
	XtVaSetValues(ttl,XtNlabel,ttl_string,NULL);
}

void ForceOrUnforce(Widget widget, XtPointer clientData, XtPointer callData)
{
	if (is_forced) {
		fwrite(unforce_string,1,strlen(unforce_string),diald_fifo);
		fflush(diald_fifo);
		is_forced = 0;
	}
	else {
		fwrite(force_string,1,strlen(force_string),diald_fifo);
		fflush(diald_fifo);
		is_forced = 1;
	}
	Redisplay();
}

void ConnectOrDisconnect(Widget widget, XtPointer clientData,
												 XtPointer callData)
{
	if (up_down) {
		fwrite(disconnect_string,1,strlen(disconnect_string),diald_fifo);
		fflush(diald_fifo);
	}
	else {
		fwrite(connect_string,1,strlen(connect_string),diald_fifo);
		fflush(diald_fifo);
	}
}

void QuitXdialdctl(Widget widget, XtPointer clientData, XtPointer callData)
{
	unlink(monitor_name);
	exit(0);
}

void IOCallback(XtPointer _pointer,int *fd,XtInputId *_input)
{
	int num_scanned;
	char string[DIALD_MONITOR_LINE_LENGTH] ="";
	char curr_ttl[] = "00:00:00";
	char max_ttl[] = "00:00:00";
	long ip_load, op_load;

	/* wait for the "STATUS" string */
	while (DIFFERENTSTRING("STATUS\n",string)) {
		fgets(string,(DIALD_MONITOR_LINE_LENGTH-1),monitor_fifo);
	}

	/* grab the status number: 1=up, 0=down */
	fgets(string,(DIALD_MONITOR_LINE_LENGTH-1),monitor_fifo);
	num_scanned=sscanf(string,"%d",&up_down);

	/* ignore the next eight lines for now */
	while (1) {
		fgets(string,(DIALD_MONITOR_LINE_LENGTH-1),monitor_fifo);
		if (SAMESTRING("QUEUE\n",string))
			break;
	}

	/* Now grab the status lines and find the longest TTL */
	while (1) {
		fgets(string,(DIALD_MONITOR_LINE_LENGTH-1),monitor_fifo);
		if (DIFFERENTSTRING(string,"END QUEUE\n")) {
			num_scanned = sscanf(string,"%*s %*s %*s %s",curr_ttl);
			if (num_scanned==1)  {
				/* string compare is conveniently sufficient */
				if (strcmp(curr_ttl,max_ttl)>0) {
					strcpy(max_ttl,curr_ttl);
				}
			}
		}
		else {
			break;
		}
	}
	sprintf(ttl_string,"TTL: %s",max_ttl);

#if HACK_FORCE_DETECTION
	/* xxx a hack to decide if the connection is forced, even though it may not
		 have been forced by this instance of xslipctl */
	if ((strcmp(max_ttl,"00:00:00")==0)&&up_down)
		XtVaSetValues(slipon,XtNbackground,bg_forced_color,NULL);
#endif

	/* get to the LOAD line (which should be the next one) */
	while (1) {
		fgets(string,(DIALD_MONITOR_LINE_LENGTH-1),monitor_fifo);
		if (SAMESTRING("LOAD\n",string))
			break;
	}
	fgets(string,(DIALD_MONITOR_LINE_LENGTH-1),monitor_fifo);
#ifdef TRY_PACKETS_SCROLLBAR
	op_load = atol(string);
#endif
	fgets(string,(DIALD_MONITOR_LINE_LENGTH-1),monitor_fifo);
#ifdef TRY_PACKETS_SCROLLBAR
	ip_load = atol(string);
#endif
	
#ifdef TRY_PACKETS_SCROLLBAR
	XawScrollbarSetThumb(inpackets,(double)0.0,(double)1.5);
#endif

	Redisplay();
}

void usage(char *name)
{
	printf("usage: %s [-f <fifo_path>]\n",name);
	exit(1);
}

Pixel name_to_color(char *name)
{
	XrmValue source, dest;
	Pixel rval;

	source.size = strlen(name);
	source.addr = name;
	dest.size = sizeof(Pixel);
	dest.addr = (XPointer) &rval;
	if (!XtConvertAndStore(toplevel,XtRString,&source,XtRPixel,&dest)) {
		rval = 0;
		printf("XtConvertAndStore returned an error. Colors may be incorrect.\n");
	}
	return rval;
}

int main(int argc, char **argv)
{
  XtAppContext app;
	int monitor_fd, diald_fifo_fd;
	char diald_control_fifo[PATH_MAX] = "/etc/diald.fifo";
	struct stat diald_fifo_stat;

	/* cheesily parse command line arguments */
	if (argc==3) {
		if (DIFFERENTSTRING(argv[1],"-f"))
			usage(argv[0]);
		strcpy(diald_control_fifo,argv[2]);
	}
	else if (argc!=1)
		usage(argv[0]);

  toplevel = XtAppInitialize(&app, "Xdialdctl",
														 (XrmOptionDescList) NULL, 0, &argc, argv,
														 (String *) NULL, (ArgList) NULL, 0);

	/* Some colors are hardcoded -- maybe they should be app-defaults, but then
		 I'd have to explain them in the manpage */
	quitxdialdctl_color = name_to_color(QUITXDIALDCTL_COLOR);
	quitxdialdctlArgs[3].value = quitxdialdctl_color;
	bg_idle_color = name_to_color(BG_IDLE_COLOR);
	bg_forced_color = name_to_color(BG_FORCED_COLOR);
	bg_connected_color = name_to_color(BG_CONNECTED_COLOR);

	boxArgs[1].value = bg_idle_color;

  box = XtCreateManagedWidget("box", boxWidgetClass, toplevel, boxArgs, XtNumber(boxArgs));

	slipon = XtCreateManagedWidget("slipon", commandWidgetClass,
																 box, sliponArgs, XtNumber(sliponArgs));
  XtAddCallback(slipon, XtNcallback, ForceOrUnforce, NULL);

	slipoff = XtCreateManagedWidget("slipoff", commandWidgetClass,
																	box, slipoffArgs, XtNumber(slipoffArgs));
  XtAddCallback(slipoff, XtNcallback, ConnectOrDisconnect, NULL);

	quitxdialdctl = XtCreateManagedWidget("quitxdialdctl", commandWidgetClass, box,
																			 quitxdialdctlArgs,
																			 XtNumber(quitxdialdctlArgs));
  XtAddCallback(quitxdialdctl, XtNcallback, QuitXdialdctl, NULL);

	ttl = XtCreateManagedWidget("ttl", commandWidgetClass, box,
															ttlArgs,
															XtNumber(ttlArgs));

#ifdef TRY_PACKETS_SCROLLBAR
	inpackets = XtCreateManagedWidget("inpackets", scrollbarWidgetClass, box,
															inpacketsArgs,
															XtNumber(inpacketsArgs));
	XawScrollbarSetThumb(inpackets,0.0,0.9);
	/* prevents scrollbar from operating like a scrollbar */
	XtVaSetValues(inpackets,XtNtranslations,XtParseTranslationTable(""),NULL);
#endif

	tmpnam(monitor_name);
	umask(0000);
	mknod(monitor_name,S_IFIFO|0666,0);
	monitor_fd = open(monitor_name,O_RDONLY|O_NONBLOCK);
	monitor_fifo = fdopen(monitor_fd,"r");

	XtAppAddInput(app,monitor_fifo->_fileno,(XtPointer)XtInputReadMask,
								IOCallback,NULL);

  XtRealizeWidget(toplevel);

	diald_fifo_fd = open(diald_control_fifo,O_WRONLY);
	if (diald_fifo_fd<0) {
		perror("xdialdctl");
	}
	fstat(diald_fifo_fd,&diald_fifo_stat);
	if ((diald_fifo_stat.st_mode&S_IFIFO)==0) {
		printf("\"%s\" must be the diald FIFO.  See the manpage for details.\n",
					 diald_control_fifo);
		exit(1);
	}
	diald_fifo = fdopen(diald_fifo_fd,"w");
	sprintf(fifo_string,"monitor %s\n",monitor_name);
	fwrite(fifo_string,1,strlen(fifo_string),diald_fifo);
	fflush(diald_fifo);

  XtAppMainLoop(app);
	return 0;
}
